      subroutine updtmet(igrd,ncol,nrow,nlay,ngas,densfac,deltat,phpt,
     &                   height,depth,pppt,press,pupt,windu,pvpt,windv,
     &                   pspt,tsurf,ptpt,tempk,pwpt,water,pkpt,rkv,conc,
     &                   adjwest,adjeast,adjsouth,adjnorth)
c 
c----CAMx v4.53 090209
c  
c     UPDTMET updates the time-varying vertical layer structure for the
c     current time step and current grid, based upon linear interpolation
c     between last input time and next input time.  Also performs
c     a similar interpolation for meteorological fields, and rescales
c     the master grid boundary concentrations for the changing met.
c 
c     Copyright 1996-2009
c     ENVIRON International Corporation
c 
c     Modifications:  
c       10/12/04   Water vapor and vertical diffusivity fields are now
c                  time-interpolated
c       10/04/05   Minimum water vapor of 1 ppm applied
c  
c     Input arguments:  
c        igrd                grid index 
c        ncol                number of columns
c        nrow                number of rows
c        nlay                number of layers
c        ngas                number of gas species
c        densfac             density conversion factor (mol/m3)
c        deltat              timestep (s)
c        phpt                time-rate change of layer interface height (m/s)
c        height              layer interface height (m)
c        pppt                time-rate change of pressure (mb/s)
c        press               presssure (mb)
c        pupt                time-rate change of u-component wind (m/s2)
c        windu               u-component wind field (m/s)
c        pvpt                time-rate change of v-component wind (m/s2)
c        windv               v-component wind field (m/s)
c        pspt                time-rate change of surface temperature (K/s)
c        tsurf               surface temperature (K)
c        ptpt                time-rate change of 3-D temperature (K/s)
c        tempk               3-D temperature (K)
c        pwpt                time-rate change of 3-D water vapor (ppm/s)
c        water               3-D water vapor (ppm)
c        pkpt                time-rate change of 3-D diffusivity (m2/s2)
c        rkv                 3-D diffusivity (m2/s)
c        conc                concentration arrray (umol/m3)
c        adjwest             adjustment factors for the West boundary
c        adjeast             adjustment factors for the East boundary
c        adjsouth            adjustment factors for the South boundary
c        adjnorth            adjustment factors for the North boundary
c             
c     Output arguments:  
c        height              layer interface height (m)
c        depth               layer depth (m) 
c        press               presssure (mb)
c        windu               u-component wind field (m/s)
c        windv               v-component wind field (m/s)
c        tsurf               surface temperature (K)
c        tempk               3-D temperature (K)
c        water               3-D water vapor (ppm)
c        rkv                 3-D diffusivity (m2/s)
c        conc                concentration arrray (umol/m3)
c             
c     Routines Called:  
c        none 
c             
c     Called by:  
c        EMISTRNS  
c 
      include 'camx.prm'
      include 'bndary.com'
c
      real height(ncol,nrow,nlay), phpt(ncol,nrow,nlay),
     &     depth(ncol,nrow,nlay), press(ncol,nrow,nlay), 
     &     pppt(ncol,nrow,nlay), windu(ncol,nrow,nlay), 
     &     pupt(ncol,nrow,nlay), windv(ncol,nrow,nlay),
     &     pvpt(ncol,nrow,nlay), tempk(ncol,nrow,nlay),
     &     ptpt(ncol,nrow,nlay), tsurf(ncol,nrow), pspt(ncol,nrow),
     &     pwpt(ncol,nrow,nlay), water(ncol,nrow,nlay),
     &     pkpt(ncol,nrow,nlay), rkv(ncol,nrow,nlay)
      real conc(ncol,nrow,nlay,ngas)
      real adjwest(MX1D,MXLAYA), adjeast(MX1D,MXLAYA),
     &     adjsouth(MX1D,MXLAYA), adjnorth(MX1D,MXLAYA)
c
c-----Entry point 
c
c-----Convert master grid boundary concs from umol/m3 to ppm
c
      if (igrd.eq.1) then
        do 20 k = 1,nlay
          do 10 j = 1,nrow
            if (ibeg(j).eq.-999) goto 10 
            i = ibeg(j) - 1 
            convfac = densfac*273./tempk(i,j,k)*press(i,j,k)/1013.
            do l = 1,ngas
              conc(i,j,k,l) = conc(i,j,k,l)/convfac
            enddo
            adjwest(j,k) = 1.0/convfac
            i = iend(j) + 1
            convfac = densfac*273./tempk(i,j,k)*press(i,j,k)/1013.
            do l = 1,ngas
              conc(i,j,k,l) = conc(i,j,k,l)/convfac
            enddo
            adjeast(j,k) = 1.0/convfac
 10       continue
          do 15 i = 1,ncol
            if (jbeg(i).eq.-999) goto 15
            j = jbeg(i) - 1
            convfac = densfac*273./tempk(i,j,k)*press(i,j,k)/1013.
            do l = 1,ngas
              conc(i,j,k,l) = conc(i,j,k,l)/convfac 
            enddo
            adjsouth(i,k) = 1.0/convfac
            j = jend(i) + 1
            convfac = densfac*273./tempk(i,j,k)*press(i,j,k)/1013.
            do l = 1,ngas
              conc(i,j,k,l) = conc(i,j,k,l)/convfac 
            enddo
            adjnorth(i,k) = 1.0/convfac
 15       continue
 20     continue
      endif
c
c-----Update vertical layer structure forward 1 timestep
c
      do 60 k = 1,nlay
        do 50 j = 1,nrow
          do 40 i = 1,ncol
            height(i,j,k) = height(i,j,k) + deltat*phpt(i,j,k)
            depth(i,j,k) = height(i,j,k)
            if (k.gt.1) depth(i,j,k) = height(i,j,k) - height(i,j,k-1)
c
c-----Update other parameters forward 1 timestep
c
            press(i,j,k) = press(i,j,k) + deltat*pppt(i,j,k)
            if (i.lt.ncol .and. j.lt.nrow) then
              windu(i,j,k) = windu(i,j,k) + deltat*pupt(i,j,k)
              windv(i,j,k) = windv(i,j,k) + deltat*pvpt(i,j,k)
            endif
            tempk(i,j,k) = tempk(i,j,k) + deltat*ptpt(i,j,k)
            water(i,j,k) = water(i,j,k) + deltat*pwpt(i,j,k)
            water(i,j,k) = amax1(water(i,j,k),1.)
            rkv(i,j,k)   = rkv(i,j,k) + deltat*pkpt(i,j,k)
            if (k.eq.1) tsurf(i,j) = tsurf(i,j) + deltat*pspt(i,j)
c
 40       continue
 50     continue
 60   continue
c
c-----Convert master grid boundary concs from ppm to umol/m3
c 
      if (igrd.eq.1) then 
        do 80 k = 1,nlay 
          do 70 j = 1,nrow 
            if (ibeg(j).eq.-999) goto 70 
            i = ibeg(j) - 1 
            convfac = densfac*273./tempk(i,j,k)*press(i,j,k)/1013. 
            do l = 1,ngas
              conc(i,j,k,l) = conc(i,j,k,l)*convfac 
            enddo
            adjwest(j,k) = adjwest(j,k) * convfac
            i = iend(j) + 1 
            convfac = densfac*273./tempk(i,j,k)*press(i,j,k)/1013. 
            do l = 1,ngas
              conc(i,j,k,l) = conc(i,j,k,l)*convfac 
            enddo
            adjeast(j,k) = adjeast(j,k) * convfac
 70       continue 
          do 75 i = 1,ncol 
            if (jbeg(i).eq.-999) goto 75 
            j = jbeg(i) - 1 
            convfac = densfac*273./tempk(i,j,k)*press(i,j,k)/1013. 
            do l = 1,ngas
              conc(i,j,k,l) = conc(i,j,k,l)*convfac 
            enddo
            adjsouth(i,k) = adjsouth(i,k) * convfac
            j = jend(i) + 1 
            convfac = densfac*273./tempk(i,j,k)*press(i,j,k)/1013. 
            do l = 1,ngas
              conc(i,j,k,l) = conc(i,j,k,l)*convfac 
            enddo
            adjnorth(i,k) = adjnorth(i,k) * convfac
 75       continue 
 80     continue 
      endif
c
      return
      end
